using SharpCharts.Base.SharpScript;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;

namespace SharpCharts.Base.Data
{
    public sealed class VolumeProfile : BarType
    {

        ProfileItem profileItem = null;

        public VolumeProfile()
        {
            this.ProfileItems = new List<ProfileItem>();
        }

        public override void Add(Bars bars, DateTime time, double open, double high, double low, double close, double bid, double ask, long volume, long openInterest, bool isRealTime)
        {
            if (bars.Count == 0)
            {
                AddBar(bars, CalculateNewBarTime(bars, time), open, high, low, close, volume, openInterest, isRealTime);
                this.profileItem = new ProfileItem();
                this.profileItem.AssignValues(close, (int)volume, bars.Instrument.InstrumentDefination.TickSize);
                this.ProfileItems.Add(profileItem);

            }
            else
            {
                if (time > bars.GetBar(bars.Count - 1).Time)
                {
                    AddBar(bars, CalculateNewBarTime(bars, time), open, high, low, close, volume, openInterest, isRealTime);
                    this.profileItem = new ProfileItem();
                    this.profileItem.AssignValues(close, (int)volume, bars.Instrument.InstrumentDefination.TickSize);
                    this.ProfileItems.Add(profileItem);
                }
                else
                {
                    
                    UpdateBar(bars, bars.GetBar(bars.Count - 1).Time, high, low, close, volume, openInterest, isRealTime);

                    if (profileItem == null)
                    {
                        throw new NullReferenceException("VolumeProfile.Add : profileItem is null");
                    }

                    this.profileItem.AssignValues(close, (int)volume, bars.Instrument.InstrumentDefination.TickSize);

                }
            }
            
        }

        private DateTime CalculateNewBarTime(Bars bars, DateTime time)
        {
           double totalMinutesDiff = bars.Session.CurrentSessionDefination.StartTime.Subtract(time.TimeOfDay).TotalMinutes % barValue;
            if (totalMinutesDiff == 0.0d)
            {
                return time;
            }
            else
            {
                return time.AddMinutes(totalMinutesDiff).AddMinutes(barValue);
            }
        }
        
		protected override void Clone(BarType clone)
		{
			//base.Clone(clone);
			
			VolumeProfile vp = clone as VolumeProfile;
			if (vp == null)
				throw new Exception("BarType mismatch while cloning");
				
			vp.BarValue = this.BarValue;
			vp.ProfileItems = new List<ProfileItem>();
		}

        public override BuiltDataType BuiltDataType
        {
            get { return Data.BuiltDataType.Tick; }
        }

        public override bool IsIntraday
        {
            get { return true; }
        }

        public override Chart.ChartStyle[] SupportedChartStyles
        {
            get { return new Chart.ChartStyle[] { new Chart.SimpleProfile() }; }
        }

        private int barValue = 30;
        [Input]
        public override int BarValue
        {
            get
            {
                return barValue;
            }
            set
            {
                barValue = Math.Max(1, value);
            }
        }

        [XmlIgnore]
        public List<ProfileItem> ProfileItems { get; private set; }
        
        

    }

    public class ProfileItem
    {
        
        public ProfileItem()
        {
            this.Max = double.MinValue;
            this.Min = double.MaxValue;
            this.Profiles = new int[0];
        }

        public double Max { get; private set; }
        public double Min { get; private set; }
        public double POC { get; private set; }
        public int POCVolume { get; private set; }

        public int[] Profiles { get; private set; }

        public void AssignValues(double price, int volume, double tickSize)
        {

            if (this.Profiles == null || this.Profiles.Length == 0)
            {
                this.Profiles = new int[] { volume };
                this.Max = this.Min = this.POC = price;
                this.POCVolume = volume;
                return;
            }

            if (price > this.Max)
            {
                int steps = (int)Math.Round((price - this.Max) / tickSize, MidpointRounding.AwayFromZero);
                this.Profiles = AddUp(steps);
                this.Profiles[0] = volume;
                if (volume > this.POCVolume)
                {
                    this.POCVolume = volume;
                    this.POC = price;
                }
                this.Max = price;
            }
            else if (price < this.Min)
            {
                int steps = (int)Math.Round((this.Min - price) / tickSize, MidpointRounding.AwayFromZero);
                this.Profiles = AddUp(steps * -1);
                if (volume > this.POCVolume)
                {
                    this.POCVolume = volume;
                    this.POC = price;
                }
                this.Profiles[this.Profiles.Length - 1] = volume;
                this.Min = price;
            }
            else
            {
                int idx = (int)Math.Round((this.Max - price) / tickSize, MidpointRounding.AwayFromZero);
    
                if (idx >= 0 && idx < this.Profiles.Length)
                {
                    this.Profiles[idx] += volume;

                    if (this.Profiles[idx] > this.POCVolume)
                    {
                        this.POCVolume = this.Profiles[idx];
                        this.POC = price;
                    }
                }
                else
                {
                    throw new IndexOutOfRangeException(string.Format("VolumeProfile.AssignValues : idx {0} out of range", idx));
                }
            }
        }

        private int[] AddUp(int steps)
        {
            int[] tmp = new int[this.Profiles.Length + Math.Abs(steps)];

            if (steps >= 0)  //price increase
            {
                this.Profiles.CopyTo(tmp, steps);
                
            }
            else
            {
                this.Profiles.CopyTo(tmp, 0);
            }
            return tmp;
        }

    }
}
